/* ===============
 * JFreeChart Demo
 * ===============
 *
 * Modified by:		Nathan Conklin (nathan@conklinfamily.net)
 * Project Info:	http://www.object-refinery.com/jfreechart/index.html
 * Project Lead:	David Gilbert (david.gilbert@object-refinery.com);
 *
 */

// Similar to com.jrefiner.chart.demo.PieChartDemo1

package com;

import java.awt.Color;
import java.util.*;
import java.applet.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;

import snap.*;

import com.jrefinery.data.DefaultCategoryDataset;
import com.jrefinery.ui.ApplicationFrame;
import com.jrefinery.chart.*;
import com.jrefinery.chart.event.*;
import com.jrefinery.chart.entity.*;

/**
 * JFreeChart's HorizontalBarChart and VerticalBarChart wrapped
 * as a Snapplet.
 */
public class BarChartSnapplet extends Snapplet 
implements Snapable, ActionListener, ChartMouseListener
{

private JPanel allPanel = new JPanel(new BorderLayout());
private JPanel jFreeChartPanel = new JPanel(new BorderLayout());
private JPanel menuPanel = new JPanel(new GridLayout(1, 2));
private JComboBox menu = new JComboBox();

private JComboBox type = new JComboBox();

private ResultSetTableModel rsModel = null;
private DefaultCategoryDataset dataset;
private int primaryKeyColumnIndex = 0;
private int selectedColumnIndex = 0;

private Object[] primaryKeys;

private static final String SELECT = "Select (fire only)";
private static final String MOUSEOVER = "MouseOver (fire only)";

private static final String VERTICAL_BARCHART_TYPE = "Vertical";
private static final String HORIZONTAL_BARCHART_TYPE = "Horizontal";

private Object lastMouseOverKey = null;

/** Returns an Enumeration of support SnapEvents */
public Enumeration getSupportedActions()
{
	Vector v = new Vector();
	v.add(SELECT);
	v.add(MOUSEOVER);
	return v.elements();
}

/** Implementation of Snapable's load() method */
public void load(ResultSet rs, String primaryKeyColumnName)
{
	rsModel = new ResultSetTableModel(rs);
	for (int i = 0; i < rsModel.getColumnCount(); i++ )
	{
		if (rsModel.getColumnName(i).equals(primaryKeyColumnName))
			primaryKeyColumnIndex = i;
	}

	if (selectedColumnIndex > rsModel.getColumnCount())
		selectedColumnIndex = 0;

	buildPopUpMenu();
	buildDataset();
	updateChart();
}

private void updateChart()
{
	// create the chart...
	JFreeChart chart;
	String typeChosen = type.getSelectedItem().toString();
	if (typeChosen.equals(VERTICAL_BARCHART_TYPE))
		chart = ChartFactory.createVerticalBarChart(
			"Vertical Bar Chart (JFreeChart)",	// chart title
			"",						// domain axis label ("Category")
			"",						// range axis label
			dataset,				// data
			false					// include legend
			);
	else
		chart = ChartFactory.createHorizontalBarChart(
			"Horizontal Bar Chart (JFreeChart)",	// chart title
			"",						// domain axis label ("Category")
			"",						// range axis label
			dataset,				// data
			false					// include legend
			);

	// set the background color for the chart...
	chart.setBackgroundPaint(Color.white);

	// add the chart to a panel...
	ChartPanel chartPanel = new ChartPanel(chart);
	chartPanel.addChartMouseListener(this);

	jFreeChartPanel.removeAll();
	jFreeChartPanel.add(chartPanel);

	allPanel.revalidate();
	allPanel.repaint();

	this.getContentPane().invalidate();
	this.getContentPane().repaint();

	this.repaint();
}

/** Builds the JFreeChart dataset from the new ResultSet */
private void buildDataset()
{
	int rowCount = rsModel.getRowCount();

	Double[][] data = new Double[rowCount][1];
	String[] seriesNames = new String[rowCount];
	String[] categories = new String[1];

	primaryKeys = new Object[rowCount];

	categories[0] = rsModel.getColumnName(selectedColumnIndex);

	for(int i = 0; i < rowCount; i++)
	{
		Object key = rsModel.getValueAt(i, primaryKeyColumnIndex);
		Object objectValue = rsModel.getValueAt(i, selectedColumnIndex);
		String stringValue = "";
		if (objectValue != null)
			stringValue = objectValue.toString();

		data[i][0] = new Double(getValue(stringValue));

		primaryKeys[i] = key;
		seriesNames[i] = key.toString();
	}

	dataset = new DefaultCategoryDataset(
		seriesNames, categories, data);
}

/** Builds the column heading selection popup menu */
private void buildPopUpMenu()
{

	menuPanel.remove(menu);

	menu = new JComboBox();                

	for (int i = 0; i < rsModel.getColumnCount(); i++ )
	{
		menu.addItem(rsModel.getColumnName(i));
	}
	menu.setSelectedIndex(selectedColumnIndex);
	menu.addActionListener(this);

	menuPanel.add(menu);

	allPanel.add(menuPanel, BorderLayout.SOUTH);

	menuPanel.revalidate();
	menuPanel.repaint();
}

/** The column to be charted has changed. */
public void actionPerformed(ActionEvent e)
{
	if (e.getSource() == menu)
	{
		String columnChosen = menu.getSelectedItem().toString();

		for (int i = 0; i < rsModel.getColumnCount(); i++ )
		{
			if (columnChosen.equals(rsModel.getColumnName(i)))
			{
				if (i != selectedColumnIndex)
				{
					selectedColumnIndex = i;
					buildDataset();
					updateChart();
					return;
				}
				else
					return;
			}
		}
	}
	else if (e.getSource() == type)
		updateChart();
}

public void chartMouseClicked(ChartMouseEvent event)
{
	chartMouseEventOccured(event, SELECT);
}

public void chartMouseMoved(ChartMouseEvent event)
{
	CategoryItemEntity entity = (CategoryItemEntity) event.getEntity();
	if (entity != null)
	{
		int keyIndex = entity.getSeries();
		Object key = primaryKeys[keyIndex];

		if (lastMouseOverKey != key)
		{	// Used to only fire mouseOver on mouseEnter events, not mouseMoved
			lastMouseOverKey = key;
			chartMouseEventOccured(event, MOUSEOVER);
		}
	}
	else
		lastMouseOverKey = null;
}

private void chartMouseEventOccured(ChartMouseEvent event, String type)
{
	CategoryItemEntity entity = (CategoryItemEntity) event.getEntity();
	if (entity != null)
	{
		int keyIndex = entity.getSeries();
		Object key = primaryKeys[keyIndex];

		Vector keys = new Vector();
		if (key != null)
			keys.add(key);

		SnapEvent snapEvent = new SnapEvent(this, type, keys);
		fireSnapEvent(snapEvent);
	}
}

/* This function will take in a String and will parse it into a double
 * as best that it can.  If it is unable, it will return 0.  */
protected double getValue(String number)
{
    String value;
    double answer = 0;
    if(number == null) return 0;

    value = number.replace('%', ' ').trim();
    try // try to parse the numeric format
    {
        answer = Double.parseDouble(value);
    }
    catch(NumberFormatException e)
    {
        if(value.length() == 0)
			answer = 0;
	else
	{
	    try  // try to parse everything before the first space
	    {
	        int space = value.indexOf(" ");
			if (space == -1)
				answer = 0;
			else
				answer = getValue(value.substring(0, space));
	    }
	    catch(Exception ex)
	    {
	        answer = 0;
	    }
	}
    }
    return answer;
}

/** Returns the component's Icon */
public Icon getIcon()
{
	return null;
}

/** Executes the appropriate action for the SnapEvent. */
public void performSnapEvent(SnapEvent e)
{
}

/** Applet's init method to create the content pane */
public void init()
{
	JLabel label = new JLabel("Empty Horizontal Bar Chart");
	jFreeChartPanel.add(label);
	allPanel.add(jFreeChartPanel);

	type.addItem(VERTICAL_BARCHART_TYPE);
	type.addItem(HORIZONTAL_BARCHART_TYPE);
	type.addActionListener(this);
	menuPanel.add(type);
	allPanel.add(menuPanel, BorderLayout.SOUTH);
	this.setContentPane(allPanel);
	updateChart();
}



}