
package scatterplot;

import java.util.Vector;

public class Stat
{

    static double[] Vtod(Vector vector)
    {
        int i = vector.size();
        Double adouble[] = new Double[i];
        vector.copyInto(adouble);
        double ad[] = new double[i];
        for(int j = 0; j < i; j++)
            ad[j] = adouble[j].doubleValue();

        return ad;
    }

    static double mean(double ad[])
    {
        int i = ad.length;
        double d = 0.0D;
        for(int j = 0; j < i; j++)
            d += ad[j];

        return d / (double)i;
    }

    static double var(double ad[])
    {
        int i = ad.length;
        double d = 0.0D;
        for(int j = 0; j < i; j++)
            d += Math.pow(ad[j], 2D);

        return (d - (double)i * Math.pow(mean(ad), 2D)) / (double)(i - 1);
    }

    static double percentile(double ad[], double d)
    {
        int i = ad.length;
        double d2 = (double)i * d;
        double ad1[] = sort(ad);
        int j = (int)d2;
        double d1;
        if(d2 > (double)j)
            d1 = ad1[j];
        else
            d1 = (ad1[j - 1] + ad1[j]) / 2D;
        return d1;
    }

    static double gammln(double d)
    {
        double ad[] = {
            76.180091729471457D, -86.505320329416776D, 24.014098240830911D, -1.231739572450155D, 0.001208650973866179D, -5.3952393849530003E-006D
        };
        double d1;
        double d2 = d1 = d;
        double d3 = d1 + 5.5D;
        d3 -= (d1 + 0.5D) * Math.log(d3);
        double d4 = 1.0000000001900149D;
        for(int i = 0; i <= 5; i++)
            d4 += ad[i] / ++d2;

        return -d3 + Math.log((2.5066282746310007D * d4) / d1);
    }

    static double betacf(double d, double d1, double d2)
    {
        double d11 = 1.0000000000000001E-030D;
        double d12 = 2.9999999999999999E-007D;
        double d13 = 100D;
        double d8 = d + d1;
        double d10 = d + 1.0D;
        double d9 = d - 1.0D;
        double d4 = 1.0D;
        double d5 = 1.0D - (d8 * d2) / d10;
        if(Math.abs(d5) < d11)
            d5 = d11;
        d5 = 1.0D / d5;
        double d7 = d5;
        int i;
        for(i = 1; (double)i <= d13; i++)
        {
            int j = 2 * i;
            double d3 = ((double)i * (d1 - (double)i) * d2) / ((d9 + (double)j) * (d + (double)j));
            d5 = 1.0D + d3 * d5;
            if(Math.abs(d5) < d11)
                d5 = d11;
            d4 = 1.0D + d3 / d4;
            if(Math.abs(d4) < d11)
                d4 = d11;
            d5 = 1.0D / d5;
            d7 *= d5 * d4;
            d3 = (-(d + (double)i) * (d8 + (double)i) * d2) / ((d + (double)j) * (d10 + (double)j));
            d5 = 1.0D + d3 * d5;
            if(Math.abs(d5) < d11)
                d5 = d11;
            d4 = 1.0D + d3 / d4;
            if(Math.abs(d4) < d11)
                d4 = d11;
            d5 = 1.0D / d5;
            double d6 = d5 * d4;
            d7 *= d6;
            if(Math.abs(d6 - 1.0D) < d12)
                break;
        }

        if((double)i > d13)
            d7 = -3000D;
        return d7;
    }

    static double betai(double d, double d1, double d2)
    {
        double d3;
        if(d2 == 0.0D || d2 == 1.0D)
            d3 = 0.0D;
        else
            d3 = Math.exp((gammln(d + d1) - gammln(d) - gammln(d1)) + d * Math.log(d2) + d1 * Math.log(1.0D - d2));
        if(d2 < (d + 1.0D) / (d + d1 + 2D))
            return (d3 * betacf(d, d1, d2)) / d;
        else
            return 1.0D - (d3 * betacf(d1, d, 1.0D - d2)) / d1;
    }

    static double tpval(double d, double d1)
    {
        return 1.0D - betai(d1 / 2D, 0.5D, d1 / (d1 + d * d));
    }

    static double max(double ad[])
    {
        int i = ad.length;
        double d = ad[0];
        for(int j = 1; j < i; j++)
            d = Math.max(d, ad[j]);

        return d;
    }

    static double min(double ad[])
    {
        int i = ad.length;
        double d = ad[0];
        for(int j = 1; j < i; j++)
            d = Math.min(d, ad[j]);

        return d;
    }

    static double[] sort(double ad[])
    {
        int i = ad.length;
        double ad1[] = new double[i];
        for(int j = 0; j < i; j++)
            ad1[j] = ad[j];

        int l = 0;
        for(int k = 1; l < ad1.length - 1 || k != 0; l++)
        {
            k = 0;
            for(int i1 = 0; i1 < ad1.length - 1; i1++)
                if(ad1[i1] > ad1[i1 + 1])
                {
                    double d = ad1[i1];
                    ad1[i1] = ad1[i1 + 1];
                    ad1[i1 + 1] = d;
                    k++;
                }

        }

        return ad1;
    }

    public Stat()
    {
    }
}
