
package snap;

import java.sql.*;
import java.beans.*;
import java.util.Enumeration;
import javax.swing.Icon;

/** The interface implemented by any Java-based visualization that
  * wants to be part of the Snap system.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  * @author Varun Saini - vsaini@vt.edu
  * @author Umer Farooq - ufarooq@vt.edu
  */
public interface Snapable
{

/** Called when Snap needs to load data into the visualization.
  *
  * @param rs the data to be loaded by the component
  * @param primaryKeyColumnName identifies the column to be used when throwing
  * and receiving SnapEvents.
  */
public void load(ResultSet rs, String primaryKeyColumnName);

/** Method used by Snap to register as a listener for SnapEvents */
public void addSnapEventListener(SnapEventListener sel);
/** Method used by Snap to unregister as a listener for SnapEvents */
public void removeSnapEventListener(SnapEventListener sel);

/** Method called when a SnapEvent is to handled by the visualization */
public void performSnapEvent(SnapEvent e);

/** Returns a list of Actions (as Strings) supported by a visualization */
public Enumeration getSupportedActions();

/** Returns the Icon for the visualization */
public Icon getIcon();

}

