
package snap.graph;

import snap.database.*;
import snap.adapter.*;
import java.util.*;

/** Graph used by the UI for manipulating coordinations.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  * @author Varun Saini - vsaini@vt.edu
  * @author Umer Farooq - ufarooq@vt.edu
  * @author Aniket Prabhune - aprabhun@vt.edu
  */
public class UIGraph extends CoordinationGraph
{

// Hard code
private Node LASTNODE = null;
private int NODECOUNT = 0;

private String emptyDescription = "";
private Query emptyQuery = new Query();

private Hashtable componentCounter = new Hashtable();

/** Assigns a unique name, generates a node, and
  * adds a new technology adapter into the graph */
public String addComponent(String componentType,
	TechnologyAdapter adapter)
{
	String componentName = null;

	try
	{
		getMonitor().acquireWrite();
		// Begin critical section
		// ----------------------------------------------------------	
			// Find a unique identifier for the component
			Integer i = (Integer) componentCounter.get(componentType);
			if( i == null )
				i = new Integer(0);

			componentName = componentType + " - " + i.toString();
			i = new Integer(i.intValue() + 1);
			componentCounter.put(componentType, i);

			// Generate the node
			UINode node = new UINode(this, componentName, componentType, 
				emptyDescription, emptyQuery, adapter);

			// Add the node to the graph
			setNode(componentName, node);
		// ----------------------------------------------------------	
		// End critical section
		getMonitor().releaseWrite();

		GraphChangeEvent gce = new
			GraphChangeEvent(this, GraphChangeEvent.NODE_ADDED, node);
		fireGraphChangeEvent(gce);
	}
	catch(InterruptedException e)
	{
		System.err.println(
			"Unable to add component to graph: " +
			e.toString());
	}

	return componentName;
}


public void removeComponent(String componentName)
{
	Node node;
	try
	{
		getMonitor().acquireWrite();
		// Begin critical section
		// ----------------------------------------------------------	
			// Add the node to the graph
			node = getNode(componentName);
			removeNode(componentName);
		// ----------------------------------------------------------	
		// End critical section
		getMonitor().releaseWrite();

		GraphChangeEvent gce = new
			GraphChangeEvent(this, GraphChangeEvent.NODE_REMOVED, node);
		fireGraphChangeEvent(gce);
	}
	catch(InterruptedException e)
	{
		System.err.println(
			"Unable to remove component to graph: " +
			e.toString());
	}
}


}
