
package table;

import javax.swing.table.AbstractTableModel;
import java.util.*;
import java.sql.*;

/** A TableModel that is constructed from a ResultSet */
public class ResultSetTableModel extends AbstractTableModel
{

Vector columnNames = new Vector();
Vector data = new Vector();

/** A TableModel that is constructed from a ResultSet */
public ResultSetTableModel(ResultSet rs)
{
	if(rs != null)
	{
		try
		{
			fillColumnNames(rs);
			fillData(rs);
		}
		catch(SQLException e)
		{
			e.printStackTrace();
		}
	}
	else
		setupEmptyTableModel();
}

/** Creates an empty table */
private void setupEmptyTableModel()
{
	columnNames.add("EmptyTable");
}

/** Fills in the column names in the TableModel */
private void fillColumnNames(ResultSet rs) throws SQLException
{
	ResultSetMetaData rsmd;
	rsmd = rs.getMetaData();

	int columnCount = rsmd.getColumnCount();
	for(int i=0; i < columnCount; i++)
	{
	   columnNames.add(rsmd.getColumnName(i+1));
	}
}

/** Fills in the data in the TableModel */
private void fillData(ResultSet rs) throws SQLException
{
	while (rs.next())
	{
		int columnCount = getColumnCount();
		Vector row = new Vector(columnCount);
		for(int i=0; i < columnCount; i++)
		{
			row.add(rs.getObject(i+1));
		}
		data.add(row);
	}
}

/** Returns the number of columns in the table */
public int getColumnCount()
{
	return columnNames.size();
}

/** Returns the number of rows in the table */
public int getRowCount()
{
	return data.size();
}

/** Returns the column name. index starts at 0 */
public String getColumnName(int index)
{
	return (String)columnNames.elementAt(index);
}

/** Returns the data item. index starts at 0 */
public Object getValueAt(int row, int column)
{
	if (row < 0 || row >= getRowCount())
		return null;
	else if (column < 0 || column >= getColumnCount() )
		return null;
	else
	{
		Vector rowData = (Vector) data.elementAt(row);
			return rowData.elementAt(column);
	}
}

/*
 * JTable uses this method to determine the default renderer/
 * editor for each cell.
 */
public Class getColumnClass(int column)
{
	if(getValueAt(0, column) == null)
		return Object.class;
	else
		return getValueAt(0, column).getClass();
}

}

