
package table;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.applet.*;
import javax.swing.*;
import java.sql.*;
import java.net.URL;

import snap.*;

/** Example of a JTable as a Snapplet.
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  */
public class TableSnapplet extends Snapplet
implements Snapable
{

private boolean inAnApplet = true;
private boolean DEBUG = true;

private ResultSet rs = null;
private String primaryKeyColumnName = null;
private int keyColumn;

private JPanel panel = new JPanel();

private ResultSetTableModel myModel = null;
private TableSorter sorter = null;
private JTable table = new JTable();

private String APPLET_TYPE = "JTable with Sorting";
private String SELECT_EVENT_TYPE = "Select";
private int DEFAULT_PRIMARY_KEY_INDEX = 0;

/** Returns an Enumeration of SnapEvents that the table can perform
  * and will fire */
public Enumeration getSupportedActions()
{
	Vector v = new Vector();
	v.add( SELECT_EVENT_TYPE );
	return v.elements();
}

/** Implementation of Snapable's load() method */
public void load(ResultSet rs, String primaryKeyColumnName)
{
    this.rs = rs;
    this.primaryKeyColumnName = primaryKeyColumnName;

    myModel = new ResultSetTableModel(rs);
    sorter = new TableSorter(myModel);
    table.setModel(sorter);
    sorter.addMouseListenerToHeaderInTable(table); 
}

/** Returns the component's Icon */
public Icon getIcon()
{
	Icon icon = null;
	try
	{
		String base = getCodeBase().toString();
		String iconURL = base + "table/table.gif";
		icon = new ImageIcon(new URL(iconURL));
	}
	catch(Exception e)
	{
		System.err.println("TableSorterDemo: " + 
			"Unable to create an icon: " + e.toString());
	}
	return icon;
}

/** Returns the Snapplet's type info */
public String getAppletInfo()
{
	return APPLET_TYPE;
}

/** Executes the appropriate action for the SnapEvent.
  * Table only supports "Select" event. */
public void performSnapEvent(SnapEvent e)
{
    if(e.getEventType().equals(SELECT_EVENT_TYPE))
    {
		keyColumn = findColumn(primaryKeyColumnName);
		Vector selectedObjects = e.getKeys();
		ListSelectionModel rowSM = table.getSelectionModel();
		rowSM.clearSelection();
    
		if(selectedObjects != null)  // Nothing is to be selected if null
		{
			// Iterate across the data elements
			for(int i = 0; i < sorter.getRowCount(); i++)
			{
				Object o = sorter.getValueAt(i, keyColumn);

				Enumeration keys = selectedObjects.elements();
				boolean selected = false;

				// Iterate across the selected keys
				while(keys.hasMoreElements() && selected == false)
				{
					Object key = keys.nextElement();
					if(o.equals(key))
					{
						// Adds the row to the set of rows that need to select
						rowSM.addSelectionInterval(i, i);
						selected = true;
					}
				}
			}
        }
    }
}

/** Finds the column index for the primaryKeyColumnName */
private int findColumn(String primaryKeyColumnName)
{
    int keyColumn;
    if(primaryKeyColumnName != null)
    {
        keyColumn = sorter.findColumn(primaryKeyColumnName);
        if(keyColumn < 0)   // Error finding the column
            keyColumn = DEFAULT_PRIMARY_KEY_INDEX; // First column
    }
    else
	{
        keyColumn = DEFAULT_PRIMARY_KEY_INDEX;  // Default to first column
	}
    return keyColumn;
}

/* Installs the selection listener.  Will fire the appropriate
 * SnapEvent when rows in the table are selected */
private void setSelectionListener()
{
    ListSelectionModel rowSM = table.getSelectionModel();
    rowSM.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e)
		{
            //Ignore extra messages.
            if (e.getValueIsAdjusting()) return;

            keyColumn = findColumn(primaryKeyColumnName);

			SnapEvent snapEvent;
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
            if (lsm.isSelectionEmpty())
			{
				snapEvent = new SnapEvent(this, SELECT_EVENT_TYPE, null);
            }
			else
			{
				int minSelectedRow = lsm.getMinSelectionIndex();
				int maxSelectedRow = lsm.getMaxSelectionIndex();
				Vector keys = new Vector();

				for(int i = minSelectedRow; i <= maxSelectedRow; i++)
				{
					if(lsm.isSelectedIndex(i))
					{
						keys.add(sorter.getValueAt(i, keyColumn));
					}
				}
				snapEvent = new SnapEvent(this, SELECT_EVENT_TYPE, keys);
            }
            fireSnapEvent(snapEvent);
        }
    });
}



/** Builds the applet's content pane that will be displayed */
private Container makeContentPane()
{
    panel.setLayout(new BorderLayout());

    load(null, null);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    setSelectionListener();

    JScrollPane scrollPane = new JScrollPane(table);
    scrollPane.revalidate();

    panel.add(scrollPane, BorderLayout.CENTER);

    return panel;
}

/** Constructor used with the component as an applications */
public TableSnapplet(boolean inAnApplet)
{
    this.inAnApplet = inAnApplet;
    if(inAnApplet)
    {
        getRootPane().putClientProperty(
            "defeatSystemEventQueueCheck", Boolean.TRUE);
    }
}

/** Default constructor with component as an applet */
public TableSnapplet()
{
	this(true);
}

/** Applet's init method to create the content pane */
public void init()
{
    setContentPane(makeContentPane());
}

/** Application's main method that creates a table in a JFrame */
public static void main(String[] args)
{
	JFrame frame = new JFrame("Table");

	frame.addWindowListener(new WindowAdapter()
	{
		public void windowClosing(WindowEvent e)
		{
			System.exit(0);
		}
	});

	TableSnapplet applet = new TableSnapplet(false);

	frame.setContentPane(applet.makeContentPane());
	frame.pack();
	frame.setVisible(true);
}

}
